using LawFirmBusinessLogic.BusinessLogics;
using LawFirmBusinessLogic.OfficePackage;
using LawFirmBusinessLogic.OfficePackage.Implements;
using LawFirmContracts.BusinessLogicContracts;
using LawFirmContracts.StorageContracts;
using LawFirmDatabaseImplement.Implements;
using LawFirmView;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using System;

namespace LawFirm
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();

            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();

            Application.Run(_serviceProvider.GetRequiredService<FormMain>());
        }

        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });
            services.AddTransient<IBlankStorage, BlankStorage>();
            services.AddTransient<IOrderStorage, OrderStorage>();
            services.AddTransient<IDocumentStorage, DocumentStorage>();
            services.AddTransient<IShopStorage, ShopStorage>();
            services.AddTransient<IBlankLogic, BlankLogic>();
            services.AddTransient<IOrderLogic, OrderLogic>();
            services.AddTransient<IDocumentLogic, DocumentLogic>();
            services.AddTransient<IReportLogic, ReportLogic>();

            services.AddTransient<IShopLogic, ShopLogic>();
            services.AddTransient<AbstractSaveToWord, SaveToWord>();
            services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
            services.AddTransient<AbstractSaveToPdf, SaveToPdf>();

            services.AddTransient<FormMain>();
            services.AddTransient<FormBlank>();
            services.AddTransient<FormBlanks>();
            services.AddTransient<FormCreateOrder>();
            services.AddTransient<FormDocument>();
            services.AddTransient<FormDocumentBlank>();
            services.AddTransient<FormDocuments>();
            services.AddTransient<FormShop>();
            services.AddTransient<FormShops>();
            services.AddTransient<FormShopSupply>();
            services.AddTransient<FormShopSell>();
            services.AddTransient<FormReportDocumentBlanks>();
            services.AddTransient<FormReportOrder>();
            services.AddTransient<FormReportShopDocuments>();
            services.AddTransient<FormReportDateOrders>();
            
        }

    }
}      
